#!/usr/bin/python
# -*- coding:utf-8; tab-width:4; mode:python -*-

import sys
import logging
import warnings
import unittest
from unittest.case import _ExpectedFailure, _UnexpectedSuccess, SkipTest

from .runner import init, Runner
from .exc import TestFailed
from .tools import StatusFilter
from .const import Status, term
from . import gvars


class PregoTestCase(object):
    def __init__(self, testcase, methodname):
        self.testcase = testcase
        self.methodname = methodname
        self.status = Status.NOEXEC

        self.name = "%s.%s" % (testcase.__class__.__name__, methodname)
        self.log = logging.getLogger(self.name)
        self.log.setLevel(logging.INFO)
        self.log.addFilter(StatusFilter(self))
        init()

    def commit(self):
        self.status = Status.UNKNOWN
        self.log.info(Status.indent('=') + term().reverse(' $name BEGIN '))
        try:
            Runner(gvars.tasks).run()
            self.status = Status.OK
        except TestFailed as test_failed:
            self.status = Status.FAIL
            # srink trackeback
            raise test_failed
        except:
            self.status = Status.ERROR
            raise
        finally:
            self.log.info('$status ' + term().reverse(' $name END '))
            init()


class TestCase(unittest.TestCase):
    def run(self, result=None):
        raised_exc = None
        orig_result = result
        if result is None:
            result = self.defaultTestResult()
            startTestRun = getattr(result, 'startTestRun', None)
            if startTestRun is not None:
                startTestRun()

        self._resultForDoCleanups = result
        result.startTest(self)

        testMethod = getattr(self, self._testMethodName)
        if (getattr(self.__class__, "__unittest_skip__", False) or
            getattr(testMethod, "__unittest_skip__", False)):
            # If the class or method was skipped.
            try:
                skip_why = (getattr(self.__class__, '__unittest_skip_why__', '')
                            or getattr(testMethod, '__unittest_skip_why__', ''))
                self._addSkip(result, skip_why)
            finally:
                result.stopTest(self)
            return
        try:
            success = False
            try:
                prego_case = PregoTestCase(self, self._testMethodName)
                self.setUp()
            except SkipTest as e:
                self._addSkip(result, str(e))
            except KeyboardInterrupt:
                raise
            except:
                result.addError(self, sys.exc_info())
            else:
                try:
                    testMethod()
                    prego_case.commit()
                except KeyboardInterrupt:
                    raise
                except self.failureException:
                    exc_info = list(sys.exc_info())
                    if exc_info[0] == TestFailed:
                        exc_info[2] = None  # remove traceback
                    result.addFailure(self, exc_info)
                except _ExpectedFailure as e:
                    addExpectedFailure = getattr(result, 'addExpectedFailure', None)
                    if addExpectedFailure is not None:
                        addExpectedFailure(self, e.exc_info)
                    else:
                        warnings.warn("TestResult has no addExpectedFailure method, reporting as passes",
                                      RuntimeWarning)
                        result.addSuccess(self)
                except _UnexpectedSuccess:
                    addUnexpectedSuccess = getattr(result, 'addUnexpectedSuccess', None)
                    if addUnexpectedSuccess is not None:
                        addUnexpectedSuccess(self)
                    else:
                        warnings.warn("TestResult has no addUnexpectedSuccess method, reporting as failures",
                                      RuntimeWarning)
                        result.addFailure(self, sys.exc_info())
                except SkipTest as e:
                    self._addSkip(result, str(e))
                except Exception as e:
                    raised_exc = e
                    result.addError(self, sys.exc_info())

                else:
                    success = True
                try:
                    self.tearDown()
#                    if not raised_exc:
#                        prego_case.commit()
                except KeyboardInterrupt:
                    raise
                except:
                    result.addError(self, sys.exc_info())
                    success = False

            cleanUpSuccess = self.doCleanups()
            success = success and cleanUpSuccess
            if success:
                result.addSuccess(self)
        finally:
            result.stopTest(self)
            if orig_result is None:
                stopTestRun = getattr(result, 'stopTestRun', None)
                if stopTestRun is not None:
                    stopTestRun()



# -*- coding:utf-8; tab-width:4; mode:python -*-

import os
import logging

from . import gvars
from .const import Status, PREGO_TMP, INDENTST
from .exc import TestFailed, TestError
from . import config
from .tools import set_testpath

log = logging.getLogger('prego')


class Runner(object):
    def __init__(self, tasks):
        self.tasks = tasks
        self.keep_running = True

    def run(self):
        self.run_tasks()
        self.wait_detached()
        self.remove_gen()
        self.report_tasks()

    def run_tasks(self):
        for t in self.tasks:
            t.run()
            if self._stop_condition(t):
                break

    def _stop_condition(self, t):
        if config.keep_going:
            return False

        return t.status.is_bad()

    def wait_detached(self):
        log.debug('%s All not detached tasks finished... killing detached',
                  Status.indent('-'))
        while 1:
            unfinished = self.get_unfinished_tasks()
            if not unfinished:
                return

            for t in reversed(unfinished):
                t.terminate()
                t.wait_detached(0.5)

    def remove_gen(self):
        log.debug("%s Temporary directory is '%s'", Status.indent(), PREGO_TMP)

        for t in self.tasks:
            t.remove_gen()

        try:
            os.removedirs(PREGO_TMP)
        except OSError:
            pass

    def report_tasks(self):
        for t in self.tasks:
            if t.status != Status.OK:
                raise TestFailed(t)

    def get_unfinished_tasks(self):
        return [t for t in self.tasks if t.thread
                and t.thread.isAlive()]


def commit(logger=None):
    result = Status.UNKNOWN
    try:
        Runner(gvars.tasks).run()
        result = Status.OK
    except TestFailed as test_failed:
        result = Status.FAIL
        # srink trackeback
        raise test_failed
    except:
        result = Status.ERROR
        raise
    finally:
        if logger:
            logger.info('%s END', result.pretty())
        init()


def run():
    try:
        commit()
    except TestFailed:
        return Status.FAIL

    return Status.OK


def init():
    del gvars.tasks[:]
    gvars.context.clear()
    set_testpath()



# -*- coding:utf-8; tab-width:4; mode:python -*-

import time
from pprint import pprint

from .const import term

from hamcrest.core.matcher import Matcher
from hamcrest.core.string_description import StringDescription
from hamcrest.core.assert_that import _assert_bool


def assert_that(arg1, arg2=None, arg3=''):
    if isinstance(arg2, Matcher):
        _assert_match(actual=arg1, matcher=arg2, reason=arg3)
    else:
        _assert_bool(assertion=arg1, reason=arg2)


def _assert_match(actual, matcher, reason):
    if not matcher.matches(actual):
        description = StringDescription()
        description.append_text(reason)     \
            .append_description_of(matcher) \
            .append_text(', but ')
        matcher.describe_mismatch(actual, description)
        raise AssertionError(str(description))


def wait_that(actual, matcher, reason='', delta=1, timeout=5):
    init = time.time()
    timeout_reached = False
    while 1:
        try:
            if time.time() - init > timeout:
                timeout_reached = True
                break

            _assert_match(actual, matcher, reason)
            break

        except AssertionError:
            time.sleep(delta)

    if timeout_reached:
        raise AssertionError('{0} was not true after {1} seconds'.format(
            matcher, timeout))



# -*- mode:python; coding:utf-8; tab-width:4 -*-

import os
from commodity.str_ import Printable


class Variable(Printable):
    def __init__(self, name):
        self.name = name

    def find(self, substr):
        return os.environ[self.name].find(substr)

    def exists(self):
        return self.name in os.environ

    def __unicode__(self):
        return u"environment variable '{0}'".format(self.name)



# -*- coding:utf-8; tab-width:4; mode:python -*-

import sys
sys.path.insert(0, '.')

import signal
import unittest

from hamcrest import contains_string, is_not
from prego import Task, context, init, commit, TestFailed


class ContextTests(unittest.TestCase):
    def test_set_read_var(self):
        init()
        context.port = 2030
        task = Task()
        cmd = task.command('echo $port')
        task.assert_that(cmd.stdout.content, contains_string('2030'))
        commit()

    def test_z_read_missing(self):
        init()
        task = Task()
        cmd = task.command('echo $port')
        task.assert_that(cmd.stdout.content, is_not(contains_string('2030')))
        commit()

    def test_default_cwd(self):
        init()
        context.cwd = '/tmp'
        task = Task()
        task.command('pwd')
        task.assert_that(task.lastcmd.stdout.content, contains_string('/tmp'))
        commit()

    def test_default_timeout(self):
        init()
        context.timeout = 3
        task = Task()
        task.command('true')

        commit()
        self.assertEquals(task.lastcmd.timeout, 3)

    # same as Signals.test_task_is_killed_by_specified_signal
    def test_default_signal(self):
        init()
        context.signal = signal.SIGINT
        task = Task()
        cmd = task.command('sleep 5', timeout=1)

        try:
            commit()
            self.fail()
        except TestFailed:
            self.assertEquals(cmd.returncode, -signal.SIGINT)



# -*- mode: python; coding: utf-8 -*-

import time

from hamcrest.core.base_matcher import BaseMatcher

from commodity.testing import assert_that
from commodity.str_ import Printable
from commodity.log import UniqueFilter
from commodity.pattern import memoizedproperty

from .const import Status, term, INDENTST
from .exc import PregoAssertionFailed, PregoAssertionError, log_traceback
from .item import DeferredItem
#from .assert_that import assert_that
from .tools import create_logger, StatusFilter


#-- assertions
class Assertion(Printable):
    def __init__(self, task, client_reason=''):
        self.task = task
        self._client_reason = client_reason

        self.name = "%s.%s" % (task.name, task.gen_assertion_index())
        task.assertions.append(self)

        self.status = Status.NOEXEC
        self._reason = '<no reason>'

    # FIXME: set private
    @memoizedproperty
    def log(self):
        retval = create_logger(self.name)
        retval.addFilter(StatusFilter(self))
        retval.addFilter(UniqueFilter())
        return retval

    def log_status(self, msg=''):
        if msg:
            msg = ': %s' % term().red_bold(msg)

        self.status.log(self.log, "$status   $name %s %s" % (str(self), msg))

    def eval(self):
        self.status = Status.UNKNOWN
        try:
            retval = self.do_eval()
        except PregoAssertionError:
            raise
        except Exception as e:
            self.status = Status.ERROR
            self.log_status()
            log_traceback(prefix='%s| ' % INDENTST)
#            self._reason = e
            raise PregoAssertionError(self)

        if not retval:
            self.status = Status.FAIL
            raise PregoAssertionFailed(self)

        self.status = Status.SOFTOK

    def terminate(self):
        pass

    def __unicode__(self):
        return self.__class__.__name__


class FailAssertion(Assertion):
    def do_eval(self):
        return False


class DeferredAssertion(Assertion):
    def __init__(self, task, actual, matcher, client_reason=''):
        self._reason = None
        Assertion.__init__(self, task, client_reason)
        self.actual = self.config_actual(actual)
        self.matcher = matcher
        self.evaluated = False
        self.caller = None

        self._reason = "%s %s" % (self.describe_actual(), matcher)

    def config_actual(self, actual):
        if isinstance(actual, DeferredItem):
            actual.config(self.task)

        return actual

    def resolve_actual(self, actual):
        if isinstance(actual, DeferredItem):
            return actual.resolve()

        return actual

    def describe_actual(self):
        try:
            return self.actual.brief()
        except AttributeError:
            return str(self.actual)

    def do_eval(self):
        actual = self.resolve_actual(self.actual)
        try:
            self.evaluated = True
            assert_that(actual, self.matcher, self._client_reason)

        except AssertionError as e:
            self._reason = u"{0} {1}".format(
                self.describe_actual(), str(e).decode('utf8'))
            return False

        return True

    def __unicode__(self):
        return u"assert that %s" % self._reason


class PollDeferredAssertion(DeferredAssertion):
    def __init__(self, task, actual, matcher, reason, delta, timeout):
        DeferredAssertion.__init__(self, task, actual, matcher, reason)
        self.timeout = timeout
        self.delta = delta

    def do_eval(self):
        tini = time.time()
        while 1:
            try:
                if time.time() - tini > self.timeout:
                    return False

                assert_that(self.resolve_actual(self.actual),
                            self.matcher, self._client_reason)

                self.log_status()
                return True

            except AssertionError:
                self.status = Status.SOFTFAIL
                time.sleep(self.delta)

    def __unicode__(self):
        return u"wait that %s" % self._reason


class Delay(Assertion):
    def __init__(self, task, n):
        Assertion.__init__(self, task)
        self.n = n

    def do_eval(self):
        time.sleep(self.n)
        return True

    def __unicode__(self):
        return u"Delay %ss" % (self.n)


#-- matchers
# FIXME: commodity
class Matcher(BaseMatcher):
#    def brief(self):
#        return u'<matcher>'

    def describe_mismatch(self, item, mismatch_description):
        mismatch_description.append_text('it is not')


class Exists(Matcher):
    def _matches(self, item):
        return item.exists()

    def describe_to(self, description):
        description.append_text('exists ')


def exists():
    return Exists()



# -*- mode:python; coding:utf-8; tab-width:4 -*-

from hamcrest import contains_string, is_not
from prego import TestCase, Task

prego_cmd = 'bin/prego-cmd.py %s -c /dev/null'


class TestOutOptions(TestCase):
    def test_print_outs_on_fail(self):
        task = Task()
        task.command(prego_cmd % '-peo examples/examples.py:Test.test_cmd_fail_with_outs', expected=1)
        task.assert_that(task.lastcmd.stderr.content,
                         contains_string("A.0.out| STDOUT"))

        task.assert_that(task.lastcmd.stderr.content,
                         contains_string("A.3.err| STDERR"))

    def test_do_not_print_outs_on_fail(self):
        task = Task()
        task.command(prego_cmd % '-p examples/examples.py:Test.test_cmd_fail_with_outs', expected=1)
        task.assert_that(task.lastcmd.stderr.content,
                         is_not(contains_string("A.0.out| STDOUT")))

        task.assert_that(task.lastcmd.stderr.content,
                         is_not(contains_string("A.3.err| STDERR")))



#!/usr/bin/python
# -*- coding:utf-8; tab-width:4; mode:python -*-

from hamcrest import is_not

from prego import Task, TestCase
from prego.debian import Package, installed


class Debian(TestCase):
    def test_installed(self):
        depends = Task()
        depends.assert_that(Package('bash'), installed())
        depends.assert_that(Package('never-installed'),
                            is_not(installed()))

    def test_installed_version(self):
        depends = Task()
        depends.assert_that(Package('python-prego'),
                            installed(min_version='0.20130206'))



# -*- coding:utf-8; tab-width:4; mode:python -*-

import os
import logging
import threading

from logging import StreamHandler
from logging.handlers import BufferingHandler
log = logging.getLogger('nose.plugins.prego')
log.addHandler(logging.StreamHandler())

import argparse

import configobj
import validate

import nose
from nose.plugins import logcapture
from nose.plugins.logcapture import FilterSet

from commodity.type_ import module_to_dict
from commodity.log import CapitalLoggingFormatter, NullHandler
from commodity.pattern import MetaBunch
from commodity.args import parser, add_argument, args

from . import config
from .tools import set_logger_default_formatter, StatusFilter, update_obj
from . import const

logging.getLogger('nose.plugins.manager').addHandler(NullHandler())
plugins_logger = logging.getLogger('nose')
plugins_logger.propagate = False

rdflog = logging.getLogger('rdflib')
# rdflog.propagate = False
rdflog.addFilter(StatusFilter())
rdflog.addHandler(NullHandler())

logging.getLogger().addFilter(StatusFilter())


class MyMemoryHandler(BufferingHandler):
    def __init__(self, capacity, logformat, logdatefmt, filters):
        BufferingHandler.__init__(self, capacity)
        fmt = CapitalLoggingFormatter(logformat, logdatefmt)
        self.setFormatter(fmt)
        self.filterset = FilterSet(filters)

        del logging.getLogger().handlers[:]

    def flush(self):
        pass  # do nothing

    def truncate(self):
        self.buffer = []

    def filter(self, record):
        return self.filterset.allow(record.name)

    def __getstate__(self):
        state = self.__dict__.copy()
        del state['lock']
        return state

    def __setstate__(self, state):
        self.__dict__.update(state)
        self.lock = threading.RLock()


logcapture.MyMemoryHandler = MyMemoryHandler
logcapture.LogCapture.logformat = '[%(levelcapital)s] %(message)s'
logcapture.clear = True


def merge_config(base, more):
#    print more, os.path.exists(more)
    base.merge(configobj.ConfigObj(more, configspec=const.PREGO_SPECS))
    base.validate(validate.Validator())


def run():
    parser.prog = 'prego'
    parser.set_specs(const.PREGO_SPECS)

    # behaviour
    add_argument('-c', '--config', metavar='FILE',
                 help='explicit config file')
    add_argument('-k', '--keep-going', action='store_true',
                 help="continue even with failed assertion or tests")
    add_argument('-d', '--dirty', action='store_true',
                 help="do not remove generated files")

    # output
    add_argument('-o', '--stdout', action='store_true',
                 help='print tests stdout')
    add_argument('-e', '--stderr', action='store_true',
                 help='print tests stderr')
    add_argument('-t', '--time-tag', dest='timetag', action='store_true',
                 help='Include time info in logs')

    add_argument('-v', '--verbose', dest='verbosity', action='count',
                 help='increase log verbosity')

    # styling
    add_argument('-p', '--plain', action='store_false', dest='color',
                 default=False,
                 help='avoid colors and styling in output')

    # pass throw nose options
    add_argument('nose', metavar='nose-args', nargs=argparse.REMAINDER)

    parser.load_config_file(const.PREGO_CMD_DEFAULTS)
    parser.parse_args()

    for x in range(args.nose.count('--')):
        args.nose.remove('--')

    if args.config:
        parser.load_config_file(os.path.abspath(args.config))
    else:
        parser.load_config_file(const.USER_CONFIG)
        parser.load_config_file(const.CWD_CONFIG)

#    print args
    update_obj(config, args)

    if config.timetag:
        logcapture.LogCapture.logformat = '%(asctime)s [%(levelcapital)s] %(message)s'

    if config.verbosity:
        args.nose += ['--nologcapture', '--quiet']

        if config.verbosity == 1:
            loglevel = logging.INFO
        elif config.verbosity >= 2:
            loglevel = logging.DEBUG
            args.nose += ['--nocapture']

        if config.verbosity > 2:
            config.stderr = config.stdout = True

        root = logging.getLogger()
        root.setLevel(loglevel)

        handler = StreamHandler()
        root.addHandler(handler)
        set_logger_default_formatter(root)

    nose.main(argv=['dummy'] + args.nose)



# -*- coding:utf-8; tab-width:4; mode:python -*-

import string
import os
import socket
import traceback
import logging

import configobj
import validate

from commodity.log import CapitalLoggingFormatter
from commodity.pattern import Bunch

from . import gvars
from .const import PREGO_TMP_BASE, PREGO_TMP, term, PREGO_SPECS

basedir = os.getcwd()


class Interpolator(object):
    fixed_vars = Bunch(dict(
        basedir      = os.path.relpath(basedir),
        fullbasedir  = os.path.abspath(basedir),
        pid          = os.getpid(),
        tmpbase      = PREGO_TMP_BASE,
        tmp          = PREGO_TMP,
        hostname     = socket.gethostname(),
        ))

    def __init__(self):
        set_testpath()
        testdir, testname  = os.path.split(gvars.testpath)

        self.vars = self.fixed_vars.copy()
        self.vars.update(
            dict(
                testdir      = os.path.relpath(testdir),
                fulltestdir  = os.path.abspath(testdir),
                testfilename = testname,
                ))
        self.vars.update(gvars.context)

    def apply(self, text):
        if not isinstance(text, basestring):
            return text

        return string.Template(text).safe_substitute(self.vars)


def create_file(fpath, *args):
    path, name = os.path.split(fpath)
    if path in [PREGO_TMP, PREGO_TMP_BASE]:
        try:
            os.makedirs(path)
        except OSError:
            pass

    return file(fpath, *args)


class PregoStreamHandler(logging.StreamHandler):
    pass


def set_logger_default_formatter(logger=None):
    logger = logger or logging.getLogger()
    formatstr = '[%(levelcapital)s] %(message)s'
    set_logger_formatter(logger, CapitalLoggingFormatter(formatstr))


def set_logger_formatter(logger, formatter):
    for handler in logger.handlers:
        handler.setFormatter(formatter)


def create_logger(name):
    retval = logging.getLogger(name)
#    retval.setLevel(logging.DEBUG)
    retval.propagate = True
    return retval


class StatusFilter(logging.Filter):
    def __init__(self, item=None):
        self.item = item
        super(StatusFilter, self).__init__()

    def filter(self, record):
        if self.item is None:
            return True

        msg = string.Template(record.msg)
        values = {}
        try:
            values['name'] = self.item.name
            values['status'] = self.item.status.pretty()
        except AttributeError:
            pass

        record.msg = "{0}{1}".format(msg.safe_substitute(values), term().normal)
        return True


def update_obj(dst, src):
    for key, value in src.items():
        setattr(dst, key, value)


def load_default_config(dest):
    config = configobj.ConfigObj(configspec=PREGO_SPECS)
    config.validate(validate.Validator())
    update_obj(dest, config['ui'])


def set_testpath():
    if gvars.testpath is None:
        return

    it = iter(traceback.extract_stack())
    for frame in it:
        if frame[3] == 'testMethod()':
            gvars.testpath = it.next()[0]



# -*- coding:utf-8; tab-width:4; mode:python -*-

import logging

from .runner import *
from .task import *
from .item import *
from .assertion import *
from .testcase import TestCase
from .const import AUTO
from .gvars import context
from .command import *

# wait_that should be in hamcrest
# from .assert_that import wait_that

from .tools import set_logger_default_formatter, load_default_config
set_logger_default_formatter(logging.getLogger())

from . import config
load_default_config(config)



# -*- mode:python; coding:utf-8; tab-width:4 -*-

from hamcrest import contains_string
from prego import TestCase, Task


class Uploaded_checks(TestCase):
    def test_pypi(self):
        URL = 'https://pypi.python.org/pypi/prego'
        task = Task()
        curl = task.command('curl ' + URL)

#        expected = '<title>prego 0.20130116</title>'  # FIXME:unicode error
        expected = '<title>prego 0.20130218'
        task.assert_that(curl.stdout.content, contains_string(expected))



#!/usr/bin/python
# -*- coding:utf-8; tab-width:4; mode:python -*-

import distutils.core
from setuptools import find_packages

distutils.core.setup(
    name             = 'prego',
    version          = '0.20130429',
    author           = 'David Villa Alises',
    author_email     = 'David.Villa@gmail.com',
    packages         = find_packages(),
    entry_points     = {
        'console_scripts': [
            'prego = prego.console:run',
            ]},
    data_files       = [
        ('', ['README.rst']),
        ('share/doc/prego', ['README.rst']),
        ('share/lib/prego', ['config.spec', 'defaults.config'])],
    url              = 'https://bitbucket.org/arco_group/prego',
    license          = 'GPLv3',
    description      = 'System test framework over POSIX shells',
    long_description = open('README.rst').read(),
    install_requires = ['PyHamcrest', 'commodity', 'blessings'],
    classifiers      = [
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development',
        'Topic :: Software Development :: Quality Assurance',
        'Topic :: Software Development :: Testing',
    ],
)



# -*- coding:utf-8; tab-width:4; mode:python -*-

import hamcrest

from prego import Task, TestCase, exists, context, running
from prego.net import Host, localhost, listen_port
from prego.shell import Variable
from prego.debian import Package, installed

from unit_tests import AssertionTest


class Net(TestCase):
    def test_localhost_listen_port(self):
        Task().assert_that(localhost, hamcrest.is_not(listen_port(9999)))

    def test_remote_host_listen_port(self):
        Task().assert_that(Host('www.google.com'), listen_port(80))

    def test_netcat(self):
        context.port = 2000
        server = Task(desc='ncat server', detach=True)
        server.assert_that(Package('nmap'), installed())
        server.assert_that(localhost,
                           hamcrest.is_not(listen_port(context.port)))
        cmd = server.command('ncat -l -p $port')
        server.assert_that(cmd.stdout.content,
                           hamcrest.contains_string('bye'))

        client = Task(desc='ncat client')
        client.wait_that(server, running())
        client.wait_that(localhost,
                         listen_port(context.port))
        client.command('ncat -c "echo bye" localhost $port')


class ShellMatchers(TestCase, AssertionTest):
    def test_variable_defined(self):
        Task().assert_that(Variable('SHELL'), exists())

    def test_variable_contains(self):
        Task().assert_that(Variable('SHELL'),
                           hamcrest.contains_string('/bin/'))


#class MailMatchers(TestCase):
#    def test_smtp(self):
#        task = Task()
#        smtp = task.smtp('default')
#        task.assert_that(stmp.send('text'), )
#
#        imap = IMAP('default')
#        task.assert_that(imap, contains_message(subject('hi')))



# -*- coding:utf-8; tab-width:4; mode:python -*-
import sys
sys.path.insert(0, '.')

import os
from unittest import TestCase
from StringIO import StringIO
from signal import SIGINT

import hamcrest
from doublex import Stub, Spy, assert_that, called

import prego
from prego import Status, gvars


class TestStatus(TestCase):
    def test_repr(self):
        self.assertEquals(str(Status.FAIL), 'FAIL')
        self.assertEquals(str(Status.OK), 'OK')
        self.assertEquals(str(Status.NOEXEC), 'NOEXEC')


class Tasks(TestCase):
    def test_zero_tasks_is_ok(self):
        prego.init()
        prego.commit()
        self.assertEquals(0, len(gvars.tasks))

    def test_one_task(self):
        prego.init()
        prego.Task().command('true')
        self.assertEquals(1, len(gvars.tasks))

    def test_init(self):
        prego.init()
        prego.Task().command('true')
        prego.init()
        self.assertEquals(0, len(gvars.tasks))

    def test_2_cmds(self):
        prego.init()
        task = prego.Task()
        task.command('true')
        task.command('ls')
        prego.commit()


class AssertionTest(object):
    def assert_that(self, actual, matcher):
        task = prego.Task()
        return task.assert_that(actual, matcher)


class FileMatchers(TestCase, AssertionTest):
    def test_file_exists(self):
        c = self.assert_that(prego.File('/etc/motd'), prego.exists())
        c.eval()

    def test_file_exists_in_cwd(self):
        prego.init()
        t = prego.Task()
        t.assert_that(prego.File('test/$testfilename'), prego.exists())
        prego.commit()

    def test_file_exists_fails(self):
        c = self.assert_that(prego.File('/etc/missing'), prego.exists())

        with self.assertRaises(prego.PregoAssertionFailed):
            c.eval()

    def test_file_do_not_exists(self):
        c = self.assert_that(prego.File('/etc/missing'),
                             hamcrest.is_not(prego.exists()))
        c.eval()

    def test_file_contains(self):
        prego.init()
        t = prego.Task()
        t.assert_that(prego.File('/etc/motd').content,
                      hamcrest.contains_string('WARRANTY'))
        prego.commit()

    def test_file_contains_fail(self):
        c = self.assert_that(prego.File('/etc/motd').content,
                             hamcrest.contains_string('missing'))

        with self.assertRaises(prego.PregoAssertionFailed):
            c.eval()


class CommandOuts(TestCase):
    def tearDown(self):
        try:
            os.remove('/tmp/task-out')
        except OSError:
            pass

    def test_stdout_with_stringio(self):
        prego.init()
        out = StringIO()
        task = prego.Task()
        task.command('echo hi', stdout=out)
        task.run()

        self.assertIn('hi', out.buflist[0])

    def test_stdout_with_filename(self):
        prego.init()
        fname = '/tmp/task-out'

        try:
            os.remove(fname)
        except OSError:
            pass

        task = prego.Task()
        task.command("echo hi", stdout=fname)
        task.assert_that(prego.File('/tmp/task-out').content,
                         hamcrest.contains_string('hi'))

        task.run()

        content = file(fname).read()

        self.assert_(os.path.exists(fname))
        self.assertIn('hi', content)

    def test_stdout_was_closed(self):
        prego.init()
        out = StringIO()
        task = prego.Task()
        task.command('echo hi', stdout=out)
        task.run()

        self.assert_(out.closed)

    def test_logging_output(self):
        prego.init()
        task = prego.Task()
        cmd = task.command('python test/utils/log.py hi')
        task.assert_that(cmd.stderr.content,
                         hamcrest.contains_string('hi'))
        task.run()

        self.assertIn('hi', cmd.stderr.read())
        self.assertEquals(Status.OK, task.status)


class OutContentMatchers(TestCase, AssertionTest):
    def test_out_contains__explicit_filename(self):
        prego.init()
        task = prego.Task()
        cmd = task.command('echo hi', stdout='/tmp/somename.out')
        task.assert_that(cmd.stdout.content,
                         hamcrest.contains_string('hi'))
        prego.commit()

    def test_out_contains__AUTO(self):
        prego.init()
        task = prego.Task()
        cmd = task.command('echo hi', stdout=prego.AUTO)
        task.assert_that(cmd.stdout.content,
                         hamcrest.contains_string('hi'))
        prego.commit()

    def test_out_contains__AUTO_wo_hamcrest_contains(self):
        task = prego.Task()
        cmd = task.command('echo hi', stdout=prego.AUTO)
        task.run()

        self.assertIn('hi', cmd.stdout.read())

    def test_out_contains__implicit(self):
        prego.init()
        task = prego.Task()
        cmd = task.command('echo hi')
        task.assert_that(cmd.stdout.content,
                         hamcrest.contains_string('hi'))
        prego.commit()

    def test_err_contains_explicit_filename(self):
        prego.init()
        task = prego.Task()
        cmd = task.command('echo hi >&2', stderr='/tmp/somename.err')
        task.assert_that(cmd.stderr.content,
                         hamcrest.contains_string('hi'))
        prego.commit()

    def test_err_contains_AUTO(self):
        prego.init()
        task = prego.Task()
        cmd = task.command('echo hi >&2', stderr=prego.AUTO)
        task.assert_that(cmd.stderr.content,
                         hamcrest.contains_string('hi'))
        prego.commit()

    def test_err_contains_AUOT_wo_hamcrest_contains(self):
        task = prego.Task()
        cmd = task.command('echo hi >&2', stderr=prego.AUTO)
        task.run()

        self.assertIn('hi', cmd.stderr.read())

    def test_err_contains__implicit(self):
        prego.init()
        task = prego.Task()
        cmd = task.command('echo hi >&2')
        task.assert_that(cmd.stderr.content,
                         hamcrest.contains_string('hi'))
        prego.commit()


class TaskMatchers(TestCase, AssertionTest):
    def test_ok(self):
        prego.init()
        task = prego.Task()
        cmd = task.command('echo hi')
        task.assert_that(cmd, prego.exits_with(0))
        prego.commit()

        self.assertEquals(Status.OK, task.status)

    def test_task_running(self):
        with Stub() as cmd:
            cmd.is_running().returns(True)
            cmd.brief().returns('STUB')

        c = self.assert_that(cmd, prego.running())
        c.eval()

    def test_task_running_fail(self):
        with Stub() as cmd:
            cmd.is_running().returns(False)
            cmd.brief().returns('STUB')

        c = self.assert_that(cmd, prego.running())
        with self.assertRaises(prego.PregoAssertionFailed):
            c.eval()


class WaitThat(TestCase):
    def test_wait_3_tries_ok(self):
        with Spy() as other_task:
            other_task.is_running().delegates([False, False, True])

        task = prego.Task()
        c = task.wait_that(other_task, prego.running(), delta=0.1, timeout=1)
        c.eval()

        assert_that(other_task.is_running, called().times(3))

    def test_wait_fail(self):
        with Spy() as cmd:
            cmd.is_running().delegates([False, False, False])

        task = prego.Task()
        c = task.wait_that(cmd, prego.running(), delta=0.1, timeout=0.3)

        with self.assertRaises(prego.PregoAssertionFailed):
            c.eval()

        assert_that(cmd.is_running, called().times(3))


class CommandReturncodes(TestCase):
    def test_no_commands_is_ok(self):
        prego.init()
        task = prego.Task()
        task.run()

        self.assertEquals(Status.OK, task.status)

    def test_ok(self):
        prego.init()
        task = prego.Task()
        task.command('echo hi')
        task.run()

        self.assertEquals(Status.OK, task.status)

    def test_fail(self):
        prego.init()
        task = prego.Task()
        task.command("false")
        task.run()

        self.assertEquals(Status.FAIL, task.status)

    def test_ls_ok(self):
        prego.init()
        task = prego.Task()
        task.command('ls')
        task.run()

        self.assertEquals(Status.OK, task.status)

    def test_ls_fail(self):
        task = prego.Task()
        task.command("ls /missing-file")
        task.run()

        self.assertEquals(Status.FAIL, task.status)


class Tasks_with_shell(TestCase):
    pass
#    def task_pipe_no_shell(self):
#        task = prego.Task('date | cat')
#        task.run()
#
#        self.assertEquals(Status.FAIL, task.status)


class PreAndPostAssertions(TestCase):
    def test_ok(self):
        task = prego.Task()
        task.command('true')
        task.run()

        self.assertEquals(Status.OK, task.status)

    def test_fail(self):
        task = prego.Task()
        task.command('false')
        task.run()

        self.assertEquals(Status.FAIL, task.status)

    def test_just_1_assertion_ok(self):
        task = prego.Task()
        task.assert_that('hello world', hamcrest.contains_string('llo'))
        task.run()

        self.assertEquals(1, len(task.assertions))
        self.assertEquals(Status.OK, task.status)

    def test_just_1_assertion_fail(self):
        task = prego.Task()
        task.assert_that('hello world', hamcrest.contains_string('missing'))
        task.run()

        self.assertEquals(Status.FAIL, task.status)
        self.assertEquals(Status.FAIL, task.assertions[0].status)

    def test_with_preassertion_fail_by_assertion(self):
        task = prego.Task()
        task.assert_that('hello world', hamcrest.contains_string('missing'))
        task.command('true')
        task.run()

        self.assertEquals(4, len(task.assertions))
        self.assertEquals(Status.FAIL, task.status)
        self.assertEquals(Status.FAIL, task.assertions[0].status)

    def test_with_postassertion_fail_by_assertion(self):
        task = prego.Task()
        task.command('true')
        task.assert_that('hello world', hamcrest.contains_string('missing'))
        task.run()

        self.assertEquals(4, len(task.assertions))
        self.assertEquals(Status.FAIL, task.status)
        self.assertEquals(Status.FAIL, task.assertions[3].status)

    def test_with_preassertion_fail_by_cmd(self):
        task = prego.Task()
        task.assert_that('hello world', hamcrest.contains_string('worl'))
        task.command('false')
        task.run()

        self.assertEquals(Status.FAIL, task.status)
        self.assertEquals(Status.OK, task.assertions[0].status)

    def test_with_postassertion_fail_by_cmd__assertion_not_executed(self):
        task = prego.Task()
        task.command('false')
        task.assert_that('hello world', hamcrest.contains_string('worl'))
        task.run()

        self.assertEquals(Status.FAIL, task.status)
        self.assertEquals(Status.NOEXEC, task.assertions[2].status)

    def test_cmd_pre_and_postassertion(self):
        task = prego.Task()
        task.assert_that('hello world', hamcrest.contains_string('worl'))
        task.command('true')
        task.assert_that(2, hamcrest.greater_than(1))
        task.run()

        self.assertEquals(Status.OK, task.status)
        self.assertEquals(Status.OK, task.assertions[0].status)
        self.assertEquals(Status.OK, task.assertions[2].status)


class CommandTimeout(TestCase):
    def test_fail_by_timeout(self):
        prego.init()
        task = prego.Task()
        task.command('sleep 2', timeout=1, expected=None)
        task.run()

        self.assertEquals(Status.FAIL, task.status)

    def test_meet_default_timeout(self):
        prego.init()
        task = prego.Task()
        task.command('sleep 2')
        prego.commit()


class ConmmandSignal(TestCase):
    def test_task_is_killed_by_specified_signal(self):
        prego.init()
        task = prego.Task()
        cmd = task.command('sleep 5', signal=SIGINT, timeout=1)

        try:
            prego.commit()
            self.fail()
        except prego.TestFailed:
            self.assertEquals(cmd.returncode, -SIGINT)


class CommandEnv(prego.TestCase):
    def test_env(self):
        task = prego.Task()
        task.command('echo $MYVAR', env={'MYVAR': '100'})
        task.assert_that(task.lastcmd.stdout.content, hamcrest.contains_string('100'))

    def test_env_no_str(self):
        task = prego.Task()
        task.command('echo $MYVAR', env={'MYVAR': 100})
        task.assert_that(task.lastcmd.stdout.content, hamcrest.contains_string('100'))


class PregoCase(prego.TestCase):
    def task_simple(self):
        task = prego.Task()
        task.command('echo hi')

# class sync_wait_that_tests(TestCase):
#     def setUp(self):
#         prego.a ssert_that(prego.localhost, hamcrest.is_not(prego.listen_port(2000)))
#         self.server = SubProcess('ncat -l 2000')
#
#     def tearDown(self):
#         self.server.terminate()
#
#     def test_wait_server_become_ready(self):
#         prego.wait_that(prego.localhost, prego.listen_port(2000))


class lastcmd(prego.TestCase):
    def test_non_exising_lastcmd(self):
        task = prego.Task()
        with self.assertRaises(IndexError):
            task.assert_that(task.lastcmd, prego.running())


class DealingWithDetach(prego.TestCase):
    def test_this(self):
        t = prego.Task('cat', detach=True)
        c = t.command("sleep 1; echo 'ready'; sleep 10",
                      expected=-15)

        o = prego.Task()
        o.wait_that(
            c.stdout.content,
            hamcrest.contains_string("ready"))

        prego.Task().delay()
